@@ -1,7 +1,7 @@
 /*
- * linux/drivers/block/ide-tape.h	Version 1.3 - ALPHA	Feb   9, 1996
+ * linux/drivers/block/ide-tape.h	Version 1.5 - ALPHA	Apr  12, 1996
  *
- * Copyright (C) 1995, 1996 Gadi Oxman <tgud@tochnapc2.technion.ac.il>
+ * Copyright (C) 1995, 1996 Gadi Oxman <gadio@netvision.net.il>
  */
 
 /*
@@ -53,7 +53,7 @@
  
 #define	IDETAPE_MIN_PIPELINE_STAGES		100
 #define	IDETAPE_MAX_PIPELINE_STAGES		200
-#define	IDETAPE_INCREASE_STAGES_RATE		0.2
+#define	IDETAPE_INCREASE_STAGES_RATE		20
 
 /*
  *	Assuming the tape shares an interface with another device, the default
@@ -97,7 +97,7 @@
 #endif
 
 /*
- *	ide-tape currently uses two continous buffers, each of the size of
+ *	ide-tape currently uses two continuous buffers, each of the size of
  *	one stage. By default, those buffers are allocated at initialization
  *	time and never released, since dynamic allocation of pages bigger
  *	than PAGE_SIZE may fail as memory becomes fragmented.
@@ -344,7 +344,7 @@ typedef struct idetape_buffer_head_s {
  */
 
 typedef struct idetape_pipeline_stage_s {
-	struct request rq;				/* The correspoding request */
+	struct request rq;				/* The corresponding request */
 	idetape_buffer_head_t *bh;			/* The data buffers */
 	struct idetape_pipeline_stage_s *next,*prev;	/* Pointers to the next and previous stages */
 } idetape_pipeline_stage_t;
@@ -479,7 +479,7 @@ typedef struct {
 	 */
 	
 	struct request *active_data_request;	/* Pointer to the request which is waiting in the device request queue */
-	char *data_buffer;			/* The correspoding data buffer (for read/write requests) */
+	char *data_buffer;			/* The corresponding data buffer (for read/write requests) */
 	int data_buffer_size;			/* Data buffer size (chosen based on the tape's recommendation */
 
 	char *merge_buffer;			/* Temporary buffer for user <-> kernel space data transfer */
