@@ -5,8 +5,6 @@
  *
  */
 
-#include <linux/module.h>
-
 #include <asm/segment.h>
 #include <asm/system.h>
 
@@ -19,6 +17,11 @@
 #include <linux/smb_fs.h>
 #include <linux/malloc.h>
 
+static inline int min(int a, int b)
+{
+	return a<b ? a : b;
+}
+
 static int 
 smb_fsync(struct inode *inode, struct file *file)
 {
@@ -250,23 +253,8 @@ struct inode_operations smb_file_inode_operations = {
 	NULL,			/* rename */
 	NULL,			/* readlink */
 	NULL,			/* follow_link */
+	NULL,			/* readpage */
+	NULL,			/* writepage */
 	NULL,			/* bmap */
 	NULL			/* truncate */
 };
-
-/*
- * Overrides for Emacs so that we follow Linus's tabbing style.
- * Emacs will notice this stuff at the end of the file and automatically
- * adjust the settings for this buffer only.  This must remain at the end
- * of the file.
- * ---------------------------------------------------------------------------
- * Local variables:
- * c-indent-level: 8
- * c-brace-imaginary-offset: 0
- * c-brace-offset: -8
- * c-argdecl-indent: 8
- * c-label-offset: -8
- * c-continued-statement-offset: 8
- * c-continued-brace-offset: 0
- * End:
- */
