@@ -15,6 +15,7 @@
 #include <linux/mm.h>
 #include <linux/tty.h>
 #include <linux/malloc.h>
+#include <linux/interrupt.h>
 
 #include <asm/segment.h>
 #include <asm/pgtable.h>
@@ -49,13 +50,33 @@ static inline void generate(unsigned long sig, struct task_struct * p)
 		wake_up_process(p);
 }
 
+/*
+ * Force a signal that the process can't ignore: if necessary
+ * we unblock the signal and change any SIG_IGN to SIG_DFL.
+ */
+void force_sig(unsigned long sig, struct task_struct * p)
+{
+	sig--;
+	if (p->sig) {
+		unsigned long mask = 1UL << sig;
+		struct sigaction *sa = p->sig->action + sig;
+		p->signal |= mask;
+		p->blocked &= ~mask;
+		if (sa->sa_handler == SIG_IGN)
+			sa->sa_handler = SIG_DFL;
+		if (p->state == TASK_INTERRUPTIBLE)
+			wake_up_process(p);
+	}
+}
+		
+
 int send_sig(unsigned long sig,struct task_struct * p,int priv)
 {
 	if (!p || sig > 32)
 		return -EINVAL;
 	if (!priv && ((sig != SIGCONT) || (current->session != p->session)) &&
-	    (current->euid ^ p->euid) && (current->euid ^ p->uid) &&
-	    (current->uid ^ p->euid) && (current->uid ^ p->uid) &&
+	    (current->euid ^ p->suid) && (current->euid ^ p->uid) &&
+	    (current->uid ^ p->suid) && (current->uid ^ p->uid) &&
 	    !suser())
 		return -EPERM;
 	if (!sig)
@@ -325,12 +346,12 @@ asmlinkage int sys_kill(int pid,int sig)
  * 
  * "I ask you, have you ever known what it is to be an orphan?"
  */
-int is_orphaned_pgrp(int pgrp)
+static int will_become_orphaned_pgrp(int pgrp, struct task_struct * ignored_task)
 {
 	struct task_struct *p;
 
 	for_each_task(p) {
-		if ((p->pgrp != pgrp) || 
+		if ((p == ignored_task) || (p->pgrp != pgrp) || 
 		    (p->state == TASK_ZOMBIE) ||
 		    (p->p_pptr->pid == 1))
 			continue;
@@ -341,6 +362,11 @@ int is_orphaned_pgrp(int pgrp)
 	return(1);	/* (sighing) "Often!" */
 }
 
+int is_orphaned_pgrp(int pgrp)
+{
+	return will_become_orphaned_pgrp(pgrp, 0);
+}
+
 static inline int has_stopped_jobs(int pgrp)
 {
 	struct task_struct * p;
@@ -367,6 +393,26 @@ static inline void forget_original_parent(struct task_struct * father)
 	}
 }
 
+static inline void close_files(struct files_struct * files)
+{
+	int i, j;
+
+	j = 0;
+	for (;;) {
+		unsigned long set = files->open_fds.fds_bits[j];
+		i = j * __NFDBITS;
+		j++;
+		if (i >= NR_OPEN)
+			break;
+		while (set) {
+			if (set & 1)
+				close_fp(files->fd[i]);
+			i++;
+			set >>= 1;
+		}
+	}
+}
+
 static inline void __exit_files(struct task_struct *tsk)
 {
 	struct files_struct * files = tsk->files;
@@ -374,14 +420,7 @@ static inline void __exit_files(struct task_struct *tsk)
 	if (files) {
 		tsk->files = NULL;
 		if (!--files->count) {
-			int i;
-			for (i=0 ; i<NR_OPEN ; i++) {
-				struct file * filp = files->fd[i];
-				if (!filp)
-					continue;
-				files->fd[i] = NULL;
-				close_fp(filp);
-			}
+			close_files(files);
 			kfree(files);
 		}
 	}
@@ -428,17 +467,17 @@ void exit_sighand(struct task_struct *tsk)
 	__exit_sighand(tsk);
 }
 
-static inline void exit_mm(void)
+static inline void __exit_mm(struct task_struct * tsk)
 {
-	struct mm_struct * mm = current->mm;
+	struct mm_struct * mm = tsk->mm;
 
 	/* Set us up to use the kernel mm state */
 	if (mm != &init_mm) {
 		flush_cache_mm(mm);
 		flush_tlb_mm(mm);
-		current->mm = &init_mm;
-		current->swappable = 0;
-		SET_PAGE_DIR(current, swapper_pg_dir);
+		tsk->mm = &init_mm;
+		tsk->swappable = 0;
+		SET_PAGE_DIR(tsk, swapper_pg_dir);
 
 		/* free the old state - not used any more */
 		if (!--mm->count) {
@@ -449,6 +488,11 @@ static inline void exit_mm(void)
 	}
 }
 
+void exit_mm(struct task_struct *tsk)
+{
+	__exit_mm(tsk);
+}
+
 /* 
  * Send signals to all our closest relatives so that they know
  * to properly mourn us..
@@ -469,7 +513,7 @@ static void exit_notify(void)
 	 */
 	if ((current->p_pptr->pgrp != current->pgrp) &&
 	    (current->p_pptr->session == current->session) &&
-	    is_orphaned_pgrp(current->pgrp) &&
+	    will_become_orphaned_pgrp(current->pgrp, current) &&
 	    has_stopped_jobs(current->pgrp)) {
 		kill_pg(current->pgrp,SIGHUP,1);
 		kill_pg(current->pgrp,SIGCONT,1);
@@ -528,7 +572,7 @@ fake_volatile:
 	del_timer(&current->real_timer);
 	sem_exit();
 	kerneld_exit();
-	exit_mm();
+	__exit_mm(current);
 	__exit_files(current);
 	__exit_fs(current);
 	__exit_sighand(current);
@@ -581,6 +625,9 @@ asmlinkage int sys_wait4(pid_t pid,unsigned int * stat_addr, int options, struct
 		if (flag)
 			return flag;
 	}
+	if (options & ~(WNOHANG|WUNTRACED|__WCLONE))
+	    return -EINVAL;
+
 	add_wait_queue(&current->wait_chldexit,&wait);
 repeat:
 	flag=0;
@@ -640,12 +687,11 @@ repeat:
 		retval = 0;
 		if (options & WNOHANG)
 			goto end_wait4;
-		current->state=TASK_INTERRUPTIBLE;
-		schedule();
-		current->signal &= ~(1<<(SIGCHLD-1));
 		retval = -ERESTARTSYS;
 		if (current->signal & ~current->blocked)
 			goto end_wait4;
+		current->state=TASK_INTERRUPTIBLE;
+		schedule();
 		goto repeat;
 	}
 	retval = -ECHILD;
