@@ -188,7 +188,7 @@ static void unix_destroy_socket(unix_socket *sk)
 		}
 		else
 		{
-			/* passed fds are erased where?? */
+			/* passed fds are erased in the kfree_skb hook */
 			kfree_skb(skb,FREE_WRITE);
 		}
 	}
@@ -354,12 +354,16 @@ static int unix_release(struct socket *sock, struct socket *peer)
 		skpair->protinfo.af_unix.locks--;	/* It may now die */
 	sk->protinfo.af_unix.other=NULL;		/* No pair */
 	unix_destroy_socket(sk);			/* Try to flush out this socket. Throw out buffers at least */
-	
+	unix_gc();					/* Garbage collect fds */	
+
 	/*
 	 *	FIXME: BSD difference: In BSD all sockets connected to use get ECONNRESET and we die on the spot. In
 	 *	Linux we behave like files and pipes do and wait for the last dereference.
 	 */
 	 
+	sock->data = NULL;
+	sk->socket = NULL;
+	
 	return 0;
 }
 
@@ -504,6 +508,7 @@ static int unix_connect(struct socket *sock, struct sockaddr *uaddr, int addr_le
 			return err;
 		skb->sk=sk;				/* So they know it is us */
 		skb->free=1;
+		skb->h.filp=NULL;
 		sk->state=TCP_CLOSE;
 		unix_mkname(sunaddr, addr_len);
 		other=unix_find_other(sunaddr->sun_path, &err);
@@ -705,10 +710,9 @@ static int unix_fd_copy(struct sock *sk, struct cmsghdr *cmsg, struct file **fp)
 	int num=cmsg->cmsg_len-sizeof(struct cmsghdr);
 	int i;
 	int *fdp=(int *)cmsg->cmsg_data;
-	num/=4;	/* Odd bytes are forgotten in BSD not errored */
-	
 
-	if(num>=UNIX_MAX_FD)
+	num /= sizeof(int);	/* Odd bytes are forgotten in BSD not errored */
+	if (num >= UNIX_MAX_FD)
 		return -EINVAL;
 	
 	/*
@@ -723,9 +727,9 @@ static int unix_fd_copy(struct sock *sk, struct cmsghdr *cmsg, struct file **fp)
 #if 0
 		printk("testing  fd %d\n", fd);
 #endif
-		if(fd < 0|| fd >=NR_OPEN)
+		if (fd < 0 || fd >= NR_OPEN)
 			return -EBADF;
-		if(current->files->fd[fd]==NULL)
+		if (current->files->fd[fd]==NULL)
 			return -EBADF;
 	}
 	
@@ -754,30 +758,6 @@ static void unix_fd_free(struct sock *sk, struct file **fp, int num)
 	}
 }
 
-/*
- *	Count the free descriptors available to a process. 
- *	Interpretation issue: Is the limit the highest descriptor (buggy
- *	allowing passed fd's higher up to cause a limit to be exceeded) -
- *	but how the old code did it - or like this...
- */
-
-static int unix_files_free(void)
-{
-	int i;
-	int n=0;
-	for (i=0;i<NR_OPEN;i++)
-	{
-		if(current->files->fd[i])
-			n++;
-	}
-	
-	i=NR_OPEN;
-	if(i>current->rlim[RLIMIT_NOFILE].rlim_cur)
-		i=current->rlim[RLIMIT_NOFILE].rlim_cur;
-	if(n>=i)
-		return 0;
-	return i-n;
-}
 
 /*
  *	Perform the AF_UNIX file descriptor pass out functionality. This
@@ -790,50 +770,39 @@ static void unix_detach_fds(struct sk_buff *skb, struct cmsghdr *cmsg)
 	/* count of space in parent for fds */
 	int cmnum;
 	struct file **fp;
-	struct file **ufp;
-	int *cmfptr=NULL;	/* =NULL To keep gcc happy */
-	/* number of fds actually passed */
+	int *cmfptr;
 	int fdnum;
-	int ffree;
-	int ufn=0;
 
-	if(cmsg==NULL)
-		cmnum=0;
-	else
+	cmfptr = NULL;
+	cmnum = 0;
+	if (cmsg)
 	{
-		cmnum=cmsg->cmsg_len-sizeof(struct cmsghdr);
-		cmnum/=sizeof(int);
-		cmfptr=(int *)&cmsg->cmsg_data;
+		cmnum = (cmsg->cmsg_len-sizeof(struct cmsghdr)) / sizeof(int);
+		cmfptr = (int *)&cmsg->cmsg_data;
 	}
 	
-	memcpy(&fdnum,skb->h.filp,sizeof(int));
-	fp=(struct file **)(skb->h.filp+sizeof(int));
-	if(cmnum>fdnum)
-		cmnum=fdnum;
-	ffree=unix_files_free();
-	if(cmnum>ffree)
-		cmnum=ffree;
-	ufp=&current->files->fd[0];
-	
+	fdnum = *(int *)skb->h.filp;
+	fp = (struct file **)(skb->h.filp+sizeof(long));
+
+	if (cmnum > fdnum)
+		cmnum = fdnum;
+
 	/*
 	 *	Copy those that fit
 	 */
-	for(i=0;i<cmnum;i++)
+	for (i = 0 ; i < cmnum ; i++)
 	{
-		/*
-		 *	Insert the fd
-		 */
-		while(ufp[ufn]!=NULL)
-			ufn++;
-		ufp[ufn]=fp[i];
-		*cmfptr++=ufn;
-		FD_CLR(ufn,&current->files->close_on_exec);
+		int new_fd = get_unused_fd();
+		if (new_fd < 0)
+			break;
+		current->files->fd[new_fd]=fp[i];
+		*cmfptr++ = new_fd;
 		unix_notinflight(fp[i]);
 	}
 	/*
 	 *	Dump those that don't
 	 */
-	for(;i<fdnum;i++)
+	for( ; i < fdnum ; i++)
 	{
 		close_fp(fp[i]);
 		unix_notinflight(fp[i]);
@@ -856,12 +825,12 @@ static void unix_destruct_fds(struct sk_buff *skb)
 static void unix_attach_fds(int fpnum,struct file **fp,struct sk_buff *skb)
 {
 
-	skb->h.filp=kmalloc(sizeof(int)+fpnum*sizeof(struct file *), 
+	skb->h.filp = kmalloc(sizeof(long)+fpnum*sizeof(struct file *), 
 							GFP_KERNEL);
 	/* number of descriptors starts block */
-	memcpy(skb->h.filp,&fpnum,sizeof(int));
+	*(int *)skb->h.filp = fpnum;
 	/* actual  descriptors */
-	memcpy(skb->h.filp+sizeof(int),fp,fpnum*sizeof(struct file *));
+	memcpy(skb->h.filp+sizeof(long),fp,fpnum*sizeof(struct file *));
 	skb->destructor = unix_destruct_fds;
 }
 
@@ -914,14 +883,14 @@ static int unix_sendmsg(struct socket *sock, struct msghdr *msg, int len, int no
 	/*
 	 *	A control message has been attached.
 	 */
-	if(msg->msg_accrights) 
+	if(msg->msg_control) 
 	{
-		struct cmsghdr *cm=unix_copyrights(msg->msg_accrights, 
-						msg->msg_accrightslen);
-		if(cm==NULL || msg->msg_accrightslen<sizeof(struct cmsghdr) ||
+		struct cmsghdr *cm=unix_copyrights(msg->msg_control, 
+						msg->msg_controllen);
+		if(cm==NULL || msg->msg_controllen<sizeof(struct cmsghdr) ||
 		   cm->cmsg_type!=SCM_RIGHTS ||
 		   cm->cmsg_level!=SOL_SOCKET ||
-		   msg->msg_accrightslen!=cm->cmsg_len)
+		   msg->msg_controllen!=cm->cmsg_len)
 		{
 			kfree(cm);
 		   	return -EINVAL;
@@ -1070,22 +1039,22 @@ static int unix_recvmsg(struct socket *sock, struct msghdr *msg, int size, int n
 	if(sk->err)
 		return sock_error(sk);
 
-	if(msg->msg_accrights) 
+	if(msg->msg_control) 
 	{
-		cm=unix_copyrights(msg->msg_accrights, 
-			msg->msg_accrightslen);
-		if(msg->msg_accrightslen<sizeof(struct cmsghdr)
+		cm=unix_copyrights(msg->msg_control, 
+			msg->msg_controllen);
+		if(msg->msg_controllen<sizeof(struct cmsghdr)
 #if 0 
 /*		investigate this further -- Stevens example doesn't seem to care */
 		||
 		   cm->cmsg_type!=SCM_RIGHTS ||
 		   cm->cmsg_level!=SOL_SOCKET ||
-		   msg->msg_accrightslen!=cm->cmsg_len
+		   msg->msg_controllen!=cm->cmsg_len
 #endif
 		)
 		{
 			kfree(cm);
-			printk("recvmsg: Bad msg_accrights\n");
+/*			printk("recvmsg: Bad msg_control\n");*/
 		   	return -EINVAL;
 		}
 	}
@@ -1158,7 +1127,7 @@ static int unix_recvmsg(struct socket *sock, struct msghdr *msg, int size, int n
 out:
 	up(&sk->protinfo.af_unix.readsem);
 	if(cm)
-		unix_returnrights(msg->msg_accrights,msg->msg_accrightslen,cm);
+		unix_returnrights(msg->msg_control,msg->msg_controllen,cm);
 	return copied;
 }
 
@@ -1305,7 +1274,7 @@ struct proto_ops unix_proto_ops = {
 
 void unix_proto_init(struct net_proto *pro)
 {
-	printk("NET3: Unix domain sockets 0.12 for Linux NET3.033.\n");
+	printk(KERN_INFO "NET3: Unix domain sockets 0.12 for Linux NET3.035.\n");
 	sock_register(unix_proto_ops.family, &unix_proto_ops);
 #ifdef CONFIG_PROC_FS
 	proc_net_register(&(struct proc_dir_entry) {
